<?php
namespace Zeedhi\Translate\Controller;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\DevOps\Model\Projectmanagement\Entities\Repository;
use Zeedhi\DevOps\Model\Translator\Entities\ZhLanguage;
use Zeedhi\DevOps\Model\Translator\Entities\ZhRepository;
use Zeedhi\DevOps\Model\Translator\Entities\ZhRepositoryLanguage;
use Zeedhi\Framework\DataSource\FilterCriteria;

class ProjectRepository extends Crud {

    protected $dataSourceName = 'projectRepository';
    protected $entityManager;

    public function __construct(Manager $dataSourceManager, $entityManager) {
        $this->entityManager = $entityManager;
        parent::__construct($dataSourceManager);
    }

    public function save(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = "projectRepositorySave";
        parent::save($request, $response);
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $oldFilterCriteria = $request->getFilterCriteria();
        $page 		= $oldFilterCriteria->getPage();
		$pageSize   = $oldFilterCriteria->getPageSize();
        $oldConditions = $oldFilterCriteria->getConditions();
        $newConditions = array_map(function( $condition ){
            if(in_array($condition['columnName'], ['GROUP', 'HOST'])){
                $condition['columnName'] = 'URL';
            }
            return $condition;
        }, $oldConditions);
		$newFilterCriteria = new FilterCriteria(
			$this->dataSourceName, 
			$newConditions, 
			$page, 
			$pageSize
		);
        
		$newRequest = new DTO\Request\Filter(
			$newFilterCriteria, 
			$request->getMethod(), 
			$request->getRoutePath(), 
			$request->getUserId()
		);
		$newRequest->setParameter('dbname', $this->dataSourceName);
        parent::find($newRequest, $response);
        $dataSet = $response->getDataSets()[0];
        $rows = array_map(function($row){
            $row['DEFAULT_LANGUAGE'] = intval($row['DEFAULT_LANGUAGE']);
            $languages = explode(',', $row['LANGUAGES']);
            $row['LANGUAGES'] = array_map('intval', $languages);
            return $row;
        }, $dataSet->getRows());
        $dataSet = new DataSet($this->dataSourceName, $rows);
        $response->setDataSets([$dataSet]);
    }

    private function deleteLanguages( $repositoryId ) {
        $repLangs = $this->entityManager->getRepository(ZhRepositoryLanguage::class)->findBy(["repository" => $repositoryId]);
        foreach($repLangs as $repLang){
            $this->entityManager->remove($repLang);
        }
    }

    private function insertRepositoryLanguage($repositoryId, $languageList) {
        $repository = $this->entityManager->getRepository(ZhRepository::class)->findOneBy(["id" => $repositoryId]);
        foreach($languageList as $languageId){
            $lang = $this->entityManager->find(ZhLanguage::class, $languageId);
            $repLang = new ZhRepositoryLanguage;
            $repLang->setLanguage($lang);
            $repLang->setRepository($repository);
            $this->entityManager->persist($repLang);
        }
    }

    public function updateLanguages(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        $operation = $row['OPERATION'] ?? 'EMPTY';
        $repositoryList = $row['REPOSITORY_LIST'] ?? 'EMPTY';
        $languages = $row['LANGUAGES'] ?? 'EMPTY';
        if(is_array($repositoryList)){
            $repositoryList = '_' . implode('_', $repositoryList) . '_';
        }
        if(!is_array($languages)){
            $languages = 'EMPTY';
        }
        $this->entityManager->beginTransaction();
        try {
            if($operation !== 'EMPTY' && $repositoryList !== 'EMPTY' && $languages !== 'EMPTY') {
                if( strtoupper($operation) == 'OVERWRITE') {
                }
                if(in_array(strtoupper($operation), ['INCLUDE', 'OVERWRITE'])) {
                    foreach($row['REPOSITORY_LIST'] as $repositoryId){
                        if(strtoupper($operation) == 'OVERWRITE') {
                            $this->deleteLanguages($repositoryId);
                        }
                        $this->insertRepositoryLanguage($repositoryId, $row['LANGUAGES']);
                    }
                    $this->entityManager->flush();
                    $this->entityManager->commit();
                }
            }
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
}