<?php
namespace Zeedhi\Translate\Controller;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\DevOps\Model\Translator\Entities\ZhLanguage;
use Zeedhi\DevOps\Model\Translator\Entities\ZhRepository;
use Zeedhi\DevOps\Model\Translator\Entities\ZhRepositoryLanguage;
use Zeedhi\Framework\DataSource\FilterCriteria;

class Repository extends \Teknisa\Libs\Controller\CRUD{

    protected $dataSourceName = 'repository';
    protected $entityManager;

    public function __construct($crudService, $environment, $entityManager) {
        $this->entityManager = $entityManager;
        parent::__construct($crudService, $environment);
    }

    protected function saveRepositoryLanguages($repository, $selectedLanguages = array()){
        $filter = array("repository" => $repository->getId());
        $repLangs = $this->entityManager->getRepository(ZhRepositoryLanguage::class)->findBy($filter);
        foreach($repLangs as $repLang){
            $id = $repLang->getLanguage()->getId();
            $selected = array_search($id, $selectedLanguages);
            if($selected === false){
                $this->entityManager->remove($repLang);
            } else {
                unset($selectedLanguages[$selected]);
            }
        }
        foreach($selectedLanguages as $languageId){
            $lang = $this->entityManager->find(ZhLanguage::class, $languageId);
            $repLang = new ZhRepositoryLanguage;
            $repLang->setLanguage($lang);
            $repLang->setRepository($repository);
            $this->entityManager->persist($repLang);
        }
    }

    public function save(DTO\Request $request, DTO\Response $response) {
        $this->entityManager->beginTransaction();
        try {
            $row = $request->getDataSet()->getRows()[0];
            $repositoryUrl = $row['URL'];
            $filter = array("url" => $repositoryUrl);
            $repository = $this->entityManager->getRepository(ZhRepository::class)->findOneBy($filter);
            if(!isset($repository)){
                $repository = new ZhRepository();
                $repository->setUrl($repositoryUrl);
            }
            
            $repository->setDefaultLanguage($row['DEFAULT_LANGUAGE']);
            $repository->setFrontPath($row['FRONT_PATH']);
            $repository->setIsPublic($row['IS_PUBLIC']);
            $repository->setIsActive($row['IS_ACTIVE']);
            $repository->setNewStructure($row['NEW_STRUCTURE']);
            $repository->setWordsPath($row['WORDS_PATH']);
            $repository->setMessagesFile($row['MESSAGES_FILE']);
            $repository->setMessagesStructure($row['MESSAGES_STRUCTURE']);
            
            $this->entityManager->persist($repository);
            if(isset($row['LANGUAGES']) && is_array($row['LANGUAGES'])) {
                $this->saveRepositoryLanguages($repository, $row['LANGUAGES']);
            }
            $this->entityManager->flush();
            $this->entityManager->commit();

            $updatedRepositories = array();
            $updatedRepositories[$repositoryUrl] = $repository->getId();
            $response->addDataSet(new DataSet($this->dataSourceName, $updatedRepositories));
        } catch(\Exception $e){
            $this->entityManager->rollback();
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $realDataSourceName = $this->dataSourceName;
        $this->dataSourceName = 'repositoryWithLanguages';
        $oldFilterCriteria = $request->getFilterCriteria();
        $page 		= $oldFilterCriteria->getPage();
		$pageSize   = $oldFilterCriteria->getPageSize();
        $oldConditions = $oldFilterCriteria->getConditions();
        $newConditions = array_map(function( $condition ){
            if(in_array($condition['columnName'], ['GROUP', 'HOST'])){
                $condition['columnName'] = 'URL';
            }
            return $condition;
        }, $oldConditions);
        
        $newFilterCriteria = new FilterCriteria(
			$this->dataSourceName, 
			$newConditions, 
			$page, 
			$pageSize
		);
        
		$newRequest = new DTO\Request\Filter(
			$newFilterCriteria, 
			$request->getMethod(), 
			$request->getRoutePath(), 
			$request->getUserId()
		);
		$newRequest->setParameter('dbname', $this->dataSourceName);
        parent::find($newRequest, $response);
        $responseRows = $response->getDataSets()[0]->getRows();
        foreach($responseRows as &$row){
            if(!is_null($row['LANGUAGES'])){
                $row['LANGUAGES'] = array_map('intval', explode(',', $row['LANGUAGES']));
            }
        }
        $response->setDataSets(array(new DataSet($realDataSourceName, $responseRows)));
    }

    public function findUnsavedRepositories(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = 'unsavedRepositories';
        $request->setParameter('dbname', $this->dataSourceName);
        parent::find($request, $response);
    }

}